@@ -1,5 +1,3 @@
-# coding: utf-8
-
 import inspect
 import os
 import re
@@ -7,6 +5,7 @@ import warnings
 from io import BufferedReader
 from io import BytesIO
 from typing import Any
+from typing import List
 from typing import Optional
 from unittest.mock import Mock
 from unittest.mock import patch
@@ -22,6 +21,7 @@ from urllib3.util.retry import Retry
 
 import responses
 from responses import BaseResponse
+from responses import Call
 from responses import CallbackResponse
 from responses import PassthroughResponse
 from responses import Response
@@ -123,7 +123,7 @@ def test_response_with_instance():
         ),
     ],
 )
-def test_replace(original, replacement):
+def test_replace(original, replacement):  # type: ignore[misc]
     @responses.activate
     def run():
         responses.add(responses.GET, "http://example.com/one", body="test1")
@@ -157,7 +157,7 @@ def test_replace(original, replacement):
         (re.compile(r"http://example\.com/one"), "http://example.com/one"),
     ],
 )
-def test_replace_error(original, replacement):
+def test_replace_error(original, replacement):  # type: ignore[misc]
     @responses.activate
     def run():
         responses.add(responses.GET, original)
@@ -203,7 +203,7 @@ def test_replace_response_object_error():
         ),
     ],
 )
-def test_upsert_replace(original, replacement):
+def test_upsert_replace(original, replacement):  # type: ignore[misc]
     @responses.activate
     def run():
         responses.add(responses.GET, "http://example.com/one", body="test1")
@@ -241,7 +241,7 @@ def test_upsert_replace(original, replacement):
         ),
     ],
 )
-def test_upsert_add(original, replacement):
+def test_upsert_add(original, replacement):  # type: ignore[misc]
     @responses.activate
     def run():
         responses.add(responses.GET, "http://example.com/one", body="test1")
@@ -299,7 +299,7 @@ def test_remove():
         ),
     ],
 )
-def test_response_equality(args1, kwargs1, args2, kwargs2, expected):
+def test_response_equality(args1, kwargs1, args2, kwargs2, expected):  # type: ignore[misc]
     o1 = BaseResponse(*args1, **kwargs1)
     o2 = BaseResponse(*args2, **kwargs2)
     assert (o1 == o2) is expected
@@ -847,7 +847,7 @@ class TestAdapters:
         "adapter_class",
         (CustomAdapter, PositionalArgsAdapter, PositionalArgsIncompleteAdapter),
     )
-    def test_custom_adapter(self, adapter_class):
+    def test_custom_adapter(self, adapter_class):  # type: ignore[misc]
         """Test basic adapter implementation and that responses can patch them properly."""
 
         @responses.activate
@@ -902,17 +902,17 @@ def test_activate_doesnt_change_signature():
 
 
 @pytest.fixture
-def my_fruit():
+def my_fruit():  # type: ignore[misc]
     return "apple"
 
 
 @pytest.fixture
-def fruit_basket(my_fruit):
+def fruit_basket(my_fruit):  # type: ignore[misc]
     return ["banana", my_fruit]
 
 
 @pytest.mark.usefixtures("my_fruit", "fruit_basket")
-class TestFixtures(object):
+class TestFixtures:
     """
     Test that pytest fixtures work well with 'activate' decorator
     """
@@ -926,7 +926,7 @@ class TestFixtures(object):
 
 def test_activate_mock_interaction():
     @patch("sys.stdout")
-    def test_function(mock_stdout):
+    def test_function(mock_stdout):  # type: ignore[misc]
         return mock_stdout
 
     decorated_test_function = responses.activate(test_function)
@@ -960,7 +960,7 @@ def test_activate_doesnt_change_signature_with_return_type():
 
 
 def test_activate_doesnt_change_signature_for_method():
-    class TestCase(object):
+    class TestCase:
         def test_function(self, a, b=None):
             return self, a, b
 
@@ -1045,7 +1045,7 @@ def test_response_cookies_multiple():
 
 @pytest.mark.parametrize("request_stream", (True, False, None))
 @pytest.mark.parametrize("responses_stream", (True, False, None))
-def test_response_cookies_session(request_stream, responses_stream):
+def test_response_cookies_session(request_stream, responses_stream):  # type: ignore[misc]
     @responses.activate
     def run():
         url = "https://example.com/path"
@@ -1100,10 +1100,10 @@ def test_response_filebody():
     def run():
         current_file = os.path.abspath(__file__)
         with responses.RequestsMock() as m:
-            with open(current_file, "r", encoding="utf-8") as out:
+            with open(current_file, encoding="utf-8") as out:
                 m.add(responses.GET, "http://example.com", body=out.read(), stream=True)
                 resp = requests.get("http://example.com", stream=True)
-            with open(current_file, "r", encoding="utf-8") as out:
+            with open(current_file, encoding="utf-8") as out:
                 assert resp.text == out.read()
 
     run()
@@ -1195,7 +1195,7 @@ def test_assert_all_requests_fired_multiple():
 def test_allow_redirects_samehost():
     redirecting_url = "http://example.com"
     final_url_path = "/1"
-    final_url = "{0}{1}".format(redirecting_url, final_url_path)
+    final_url = f"{redirecting_url}{final_url_path}"
     url_re = re.compile(r"^http://example.com(/)?(\d+)?$")
 
     def request_callback(request):
@@ -1209,7 +1209,7 @@ def test_allow_redirects_samehost():
                 n = 1
             else:
                 n = 0
-            redirect_headers = {"location": "/{0!s}".format(n)}
+            redirect_headers = {"location": f"/{n!s}"}
             return 301, redirect_headers, None
 
     def run():
@@ -1332,6 +1332,29 @@ def test_headers():
     assert_reset()
 
 
+def test_headers_deduplicated_content_type():
+    """Test to ensure that we do not have two values for `content-type`.
+
+    For more details see https://github.com/getsentry/responses/issues/644
+    """
+
+    @responses.activate
+    def run():
+        responses.get(
+            "https://example.org/",
+            json={},
+            headers={"Content-Type": "application/json"},
+        )
+        responses.start()
+
+        resp = requests.get("https://example.org/")
+
+        assert resp.headers["Content-Type"] == "application/json"
+
+    run()
+    assert_reset()
+
+
 def test_content_length_error(monkeypatch):
     """
     Currently 'requests' does not enforce content length validation,
@@ -1361,15 +1384,13 @@ def test_content_length_error(monkeypatch):
     # Type errors here and on 1250 are ignored because the stubs for requests
     # are off https://github.com/python/typeshed/blob/f8501d33c737482a829c6db557a0be26895c5941
     #   /stubs/requests/requests/packages/__init__.pyi#L1
-    original_init = getattr(requests.packages.urllib3.HTTPResponse, "__init__")  # type: ignore
+    original_init = getattr(urllib3.HTTPResponse, "__init__")
 
     def patched_init(self, *args, **kwargs):
         kwargs["enforce_content_length"] = True
         original_init(self, *args, **kwargs)
 
-    monkeypatch.setattr(
-        requests.packages.urllib3.HTTPResponse, "__init__", patched_init  # type: ignore
-    )
+    monkeypatch.setattr(urllib3.HTTPResponse, "__init__", patched_init)
 
     run()
     assert_reset()
@@ -1696,12 +1717,12 @@ class TestPassthru:
         @responses.activate
         def run():
             responses.add(PassthroughResponse(responses.GET, url))
-            responses.add(responses.GET, "{}/one".format(url), body="one")
+            responses.add(responses.GET, f"{url}/one", body="one")
             responses.add(responses.GET, "http://example.com/two", body="two")
 
             resp = requests.get("http://example.com/two")
             assert_response(resp, "two")
-            resp = requests.get("{}/one".format(url))
+            resp = requests.get(f"{url}/one")
             assert_response(resp, "one")
             resp = requests.get(url)
             assert_response(resp, "OK")
@@ -1762,12 +1783,12 @@ class TestPassthru:
         @responses.activate
         def run():
             responses.add_passthru(url)
-            responses.add(responses.GET, "{}/one".format(url), body="one")
+            responses.add(responses.GET, f"{url}/one", body="one")
             responses.add(responses.GET, "http://example.com/two", body="two")
 
             resp = requests.get("http://example.com/two")
             assert_response(resp, "two")
-            resp = requests.get("{}/one".format(url))
+            resp = requests.get(f"{url}/one")
             assert_response(resp, "one")
             resp = requests.get(url)
             assert_response(resp, "OK")
@@ -1784,7 +1805,7 @@ class TestPassthru:
         @responses.activate
         def run():
             responses.add_passthru(re.compile(f"{url}/\\w+"))
-            responses.add(responses.GET, "{}/one".format(url), body="one")
+            responses.add(responses.GET, f"{url}/one", body="one")
             responses.add(responses.GET, "http://example.com/two", body="two")
 
             resp = requests.get("http://example.com/two")
@@ -1840,6 +1861,34 @@ class TestPassthru:
         run()
         assert_reset()
 
+    def test_real_send_argument(self):
+        def run():
+            # the following mock will serve to catch the real send request from another mock and
+            # will "donate" `unbound_on_send` method
+            mock_to_catch_real_send = responses.RequestsMock(
+                assert_all_requests_are_fired=True
+            )
+            mock_to_catch_real_send.post(
+                "http://send-this-request-through.com", status=500
+            )
+
+            with responses.RequestsMock(
+                assert_all_requests_are_fired=True,
+                real_adapter_send=mock_to_catch_real_send.unbound_on_send(),
+            ) as r_mock:
+                r_mock.add_passthru("http://send-this-request-through.com")
+
+                r_mock.add(responses.POST, "https://example.org", status=200)
+
+                response = requests.post("https://example.org")
+                assert response.status_code == 200
+
+                response = requests.post("http://send-this-request-through.com")
+                assert response.status_code == 500
+
+        run()
+        assert_reset()
+
 
 def test_method_named_param():
     @responses.activate
@@ -1870,13 +1919,13 @@ def test_custom_target(monkeypatch):
         "http://example.com/other/path/",
     ),
 )
-def test_request_param(url):
+def test_request_param(url):  # type: ignore[misc]
     @responses.activate
     def run():
         params = {"hello": "world", "example": "params"}
         responses.add(
             method=responses.GET,
-            url="{0}?hello=world".format(url),
+            url=f"{url}?hello=world",
             body="test",
             match_querystring=False,
         )
@@ -1913,7 +1962,7 @@ def test_request_param_with_multiple_values_for_the_same_key():
 @pytest.mark.parametrize(
     "url", ("http://example.com", "http://example.com?hello=world")
 )
-def test_assert_call_count(url):
+def test_assert_call_count(url):  # type: ignore[misc]
     @responses.activate
     def run():
         responses.add(responses.GET, url)
@@ -1923,7 +1972,7 @@ def test_assert_call_count(url):
 
         with pytest.raises(AssertionError) as excinfo:
             responses.assert_call_count(url, 2)
-        assert "Expected URL '{0}' to be called 2 times. Called 0 times.".format(
+        assert "Expected URL '{}' to be called 2 times. Called 0 times.".format(
             url
         ) in str(excinfo.value)
 
@@ -1936,7 +1985,7 @@ def test_assert_call_count(url):
         requests.get(url)
         with pytest.raises(AssertionError) as excinfo:
             responses.assert_call_count(url, 3)
-        assert "Expected URL '{0}' to be called 3 times. Called 2 times.".format(
+        assert "Expected URL '{}' to be called 3 times. Called 2 times.".format(
             url
         ) in str(excinfo.value)
 
@@ -1988,6 +2037,70 @@ def test_call_count_without_matcher():
     assert_reset()
 
 
+def test_response_calls_indexing_and_slicing():
+    @responses.activate
+    def run():
+        responses.add(responses.GET, "http://www.example.com")
+        responses.add(responses.GET, "http://www.example.com/1")
+        responses.add(responses.GET, "http://www.example.com/2")
+
+        requests.get("http://www.example.com")
+        requests.get("http://www.example.com/1")
+        requests.get("http://www.example.com/2")
+        requests.get("http://www.example.com/1")
+        requests.get("http://www.example.com")
+
+        # Use of a type hints here ensures mypy knows the difference between index and slice.
+        individual_call: Call = responses.calls[0]
+        call_slice: List[Call] = responses.calls[1:-1]
+
+        assert individual_call.request.url == "http://www.example.com/"
+
+        assert call_slice == [
+            responses.calls[1],
+            responses.calls[2],
+            responses.calls[3],
+        ]
+        assert [c.request.url for c in call_slice] == [
+            "http://www.example.com/1",
+            "http://www.example.com/2",
+            "http://www.example.com/1",
+        ]
+
+    run()
+    assert_reset()
+
+
+def test_response_calls_and_registry_calls_are_equal():
+    @responses.activate
+    def run():
+        rsp1 = responses.add(responses.GET, "http://www.example.com")
+        rsp2 = responses.add(responses.GET, "http://www.example.com/1")
+        rsp3 = responses.add(
+            responses.GET, "http://www.example.com/2"
+        )  # won't be requested
+
+        requests.get("http://www.example.com")
+        requests.get("http://www.example.com/1")
+        requests.get("http://www.example.com")
+
+        assert len(responses.calls) == len(rsp1.calls) + len(rsp2.calls) + len(
+            rsp3.calls
+        )
+        assert rsp1.call_count == 2
+        assert len(rsp1.calls) == 2
+        assert rsp1.calls[0] is responses.calls[0]
+        assert rsp1.calls[1] is responses.calls[2]
+        assert rsp2.call_count == 1
+        assert len(rsp2.calls) == 1
+        assert rsp2.calls[0] is responses.calls[1]
+        assert rsp3.call_count == 0
+        assert len(rsp3.calls) == 0
+
+    run()
+    assert_reset()
+
+
 def test_fail_request_error():
     """
     Validate that exception is raised if request URL/Method/kwargs don't match
@@ -2050,7 +2163,7 @@ def test_fail_request_error():
         ),
     ],
 )
-def test_response_representations(response_params, expected_representation):
+def test_response_representations(response_params, expected_representation):  # type: ignore[misc]
     response = Response(**response_params)
 
     assert str(response) == expected_representation
@@ -2096,7 +2209,7 @@ def test_mocked_responses_list_registered():
         ("http://fizzbuzz/foo", "http://fizzbuzz/foo"),
     ],
 )
-def test_rfc_compliance(url, other_url):
+def test_rfc_compliance(url, other_url):  # type: ignore[misc]
     @responses.activate
     def run():
         responses.add(method=responses.GET, url=url)
@@ -2204,7 +2317,7 @@ class TestStrictWrapper:
         run_not_strict()
 
     @pytest.mark.parametrize("assert_fired", (True, False, None))
-    def test_nested_decorators(self, assert_fired):
+    def test_nested_decorators(self, assert_fired):  # type: ignore[misc]
         """Validate if assert_all_requests_are_fired is applied from the correct function.
 
         assert_all_requests_are_fired must be applied from the function
@@ -2497,7 +2610,7 @@ class TestMaxRetry:
         assert_reset()
 
     @pytest.mark.parametrize("raise_on_status", (True, False))
-    def test_max_retries_exceed(self, raise_on_status):
+    def test_max_retries_exceed(self, raise_on_status):  # type: ignore[misc]
         @responses.activate(registry=registries.OrderedRegistry)
         def run():
             url = "https://example.com"
